#!/usr/bin/env ruby
###########################################################################
###########################################################################
##  Copyright The Resistance (C) 1995-2011                               ##
##  Author: blass                                                        ##
##  blass[at]sdf.lonestar.org                                            ##
##  Badass 0.2 Beta                                                      ##
##  This file is part of Badass.                                         ##
##  Badass is free software: you can redistribute it and/or modify       ##
##  it under the terms of the GNU General Public License as published by ##
##  the Free Software Foundation, either version 3 of the License, or    ##
##  (at your option) any later version.                                  ##
##  Badass is distributed in the hope that it will be useful,            ##
##  but WITHOUT ANY WARRANTY; without even the implied warranty of       ##
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        ##
##  GNU General Public License for more details.                         ##
##  You should have received a copy of the GNU General Public License    ##
##  along with Badass. If not, see http://www.gnu.org/licenses/          ##
###########################################################################
###########################################################################

module Racket
  module L3
    # CDP -- Cisco Discovery Protocol
    # http://www.cisco.biz/univercd/cc/td/doc/product/lan/trsrb/frames.htm#xtocid12
    class CDP < RacketPart
      # CDP Version (generally 1)
      unsigned :version, 8, { :default => 1 }
      # Time-to-live of the data in this message
      unsigned :ttl, 8
      # Checksum
      unsigned :checksum, 16
      # Payload of this CDP message.  Generally untouched.
      rest :payload
      
      
      def initialize(*args)
        super(*args)
      end
      
      # Add a new field to this CDP message.
      def add_field(type, value) 
        t = Racket::Misc::TLV.new(2,2)
        t.type = type
        t.value = value
        t.length = 4 + value.length
        self.payload += t.encode   
      end 
      
      # Check the checksum for this IP datagram
      def checksum? 
        self.checksum == compute_checksum
      end 
      
      # Compute and set the checksum for this IP datagram
      def checksum! 
        self.checksum = compute_checksum
      end 
      
      # Fix this CDP message up for sending.
      def fix! 
        self.checksum!
      end 
      
      private
      
      # Compute the checksum for this IP datagram
      def compute_checksum
        pseudo = []
        pseudo << ((self.version << 8) | self.ttl)
        pseudo << 0
        pseudo << self.payload
        L3::Misc.checksum(pseudo.pack("nna*"))
      end 
      
    end
  end
end
# vim: set ts=2 et sw=2:
